#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <mpi.h>
#include <complex.h>
#include <math.h>
//#include <fftw3.h>
#include "2dTEpbc.h"
#include "physical_constants.h"



int main(int argc, char** argv)
{
  FILE *fpt,*fpt2,*fpt3;
  float *Ex,*Ey,*Hzx,*Hzy;			      // 2D TE mode fields
  float *Fx,*Fy;				      // Time response functions
  float *sx,*sy,*sigmax,*sigmay,*eperm,*einf,*Wp,*r;  // Optical constants
  float s_max,s_may;
  float dx,dy,dt;				      // dx dy dt
  float *Px,*Py;				      // electric dipole moment
  float *sE,*sHz,*rE,*rHz;			      // dummy field for MPI sync.
  float freq,lambda;
  float kx=0,dk=0,ks=0,kf=0,k=0,R=0;
  char filename1[256],filename2[256],filename3[256];

  int nx,ny,npml;				      // Cal. space indexs
  int max_iter,iter,i,j;			      // dummy index for loop
  int node,size;
  int det1x,det1y;
  //fftw_complex *field,*out;

  MPI_Status status;

  MPI_Init(&argc,&argv);
  MPI_Comm_rank(MPI_COMM_WORLD,&node);
  MPI_Comm_size(MPI_COMM_WORLD,&size);


  nx=57;
  ny=1016;
  npml=8;
  dx=5e-9;
  dy=5e-9;
  max_iter = 131072;
  lambda = 619.9e-9;
  det1x = (nx/2 + 2);
  det1y = (ny/2 + 8);
  R = dx*(nx-1);
  ks = 0.06;
  kf = 0.1;
  dk = 0.01;

  ks = ks*2*M_PI/R;
  kf = kf*2*M_PI/R;
  dk = dk*2*M_PI/R;


  dt=min(dx,dy)/( sqrtf(2.1)*LIGHT_SPEED );

  if(node == 0)
  {
    printf("dx=%e\n",dx);
    printf("dy=%e\n",dy);
    printf("dt=%e\n",dt);
    printf("R=%e\n",R);
    printf("k_start=%f\n",ks*R/2/M_PI);
    printf("k_end=%f\n",kf*R/2/M_PI);
  }




  freq = LIGHT_SPEED/lambda;

  for(k=ks;k<=kf;k=k+dk)
  {
    INIT;
    if(node == 0)
    {
      printf("current kx = %f\n",k*R/2/M_PI);
      sprintf(filename1,"%f_p1.dat",k*R/2/M_PI);
      sprintf(filename2,"%f_p2.dat",k*R/2/M_PI);
      sprintf(filename3,"%f_p3.dat",k*R/2/M_PI);
      fpt = fopen(filename1,"w");
      fpt2 = fopen(filename2,"w");
      fpt3 = fopen(filename3,"w");
    }
    if(node == 0)
      printf("Initializing Perfect Matched Layer...........");
    SET_PML;
    if(node == 0)
      printf("OK\n");

    if(node == 0)
      printf("Making a computational domain.........");
    ////////////////////////////////////
    // Construct the calculation space
    ////////////////////////////////////
    for(i=0;i<nx;i++)
      for(j=0;j<ny;j++)
      {
	if( j<ny/2  )
	{
	  sigmax[p(i,j)] = 0.0;
	  sigmay[p(i,j)] = 0.0;
	  eperm[p(i,j)] = 1*EPSILON_0;
	  Wp[p(i,j)] = 6.7320e15;
	  r[p(i,j)] = 0.0;//9.7269e+13;
	}
	if(j>=ny/2-10 && j<ny/2 && i>=(nx-1)/2-20 && i<(nx-1)/2+17)
	{
	  eperm[p(i,j)] = 2.1316*EPSILON_0;
	  Wp[p(i,j)] = 0.0;
	  r[p(i,j)] = 0.0;//9.7269e+13;
	}
	if(j>=ny/2)
	  eperm[p(i,j)] = 6.5025*EPSILON_0;
      }
    if(node == 0)
      printf("OK\n");
    // Iteration start
    for(iter=0;iter<max_iter;iter++)
    {
      if(iter == 0)
      {
	Py[p(det1x,det1y)] = 1.0;
	Px[p(det1x,det1y)] = 1.0;
	Py[p(det1x+1,det1y+1)] = 1.0;
	Px[p(det1x+1,det1y+1)] = -1.0;
      }
      else
      {
	Py[p(det1x,det1y)] = 0;
	Px[p(det1x,det1y)] = 0;
	Py[p(det1x+1,det1y+1)] = 0;
	Px[p(det1x+1,det1y+1)] = 0;
      }



      // E field update
      for(i=1;i<nx;i++)
	for(j=0;j<ny;j++){
	  E_FIELD_UPDATE;
	}
      E_FIELD_SYNC;
      for(j=0;j<ny;j++){
	E_FIELD_BC_UPDATE;
      }

      // H field update
      for(i=0;i<nx-1;i++)
	for(j=0;j<ny;j++){
	  H_FIELD_UPDATE;
	}
      H_FIELD_SYNC;
      for(j=0;j<ny;j++){
	H_FIELD_BC_UPDATE;
      }

      // Response function update
      for(i=0;i<nx;i++)
	for(j=0;j<ny;j++){
	  F_FUNCTION_UPDATE;
	}
      if(node == 0)
      {
	fprintf(fpt,"%e\t%e\n",iter*dt,Hzx[p((nx-1)/2,ny/2+10)]+Hzy[p((nx-1)/2,ny/2+10)]);
	fprintf(fpt2,"%e\t%e\n",iter*dt,Hzx[p(20,ny/2+1)]+Hzy[p(20,ny/2+1)]);
	fprintf(fpt3,"%e\t%e\n",iter*dt,Hzx[p(20,900)]+Hzy[p(20,900)]);
      }
    }// end of iteration
    if(node == 0)
    {
      fclose(fpt); 
      fclose(fpt2);
      fclose(fpt3);
    }
  }// end of k iteration loop

  if(node == 0) 
  {
    fpt = fopen("Hz.dat","w");
    for(i=0;i<nx;i++)
    {
      for(j=npml;j<ny-npml;j++)
	//fprintf(fpt,"%e\t",Hzx[p(i,j)]+Hzy[p(i,j)]);
	fprintf(fpt,"%e\t",eperm[p(i,j)]);
      fprintf(fpt,"\n");
    }
    fclose(fpt);
  }


  MPI_Finalize();

  return 0;
}
